/*
 * Copyright 2019 Confluent Inc.
 *
 * Licensed under the Confluent Community License (the "License"); you may not use
 * this file except in compliance with the License.  You may obtain a copy of the
 * License at
 *
 * http://www.confluent.io/confluent-community-license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 */

package io.confluent.kafka.secretregistry.storage;

import static io.confluent.kafka.secretregistry.storage.SecretKey.compareKeys;

import com.fasterxml.jackson.annotation.JsonProperty;

import java.util.Objects;

import javax.validation.constraints.Min;

import org.hibernate.validator.constraints.NotEmpty;

import io.confluent.kafka.secretregistry.crypto.Encrypted;

public class SecretValue implements Comparable<SecretValue>, SecretRegistryValue {

  @NotEmpty
  private String path;
  @NotEmpty
  private String key;
  @Min(1)
  private Integer version;
  @NotEmpty
  private Encrypted encrypted;
  @NotEmpty
  private String checksum;
  private String createdBy;
  @NotEmpty
  private Long createdAt;

  public SecretValue(
      @JsonProperty("path") String path,
      @JsonProperty("key") String key,
      @JsonProperty("version") Integer version,
      @JsonProperty("encrypted") Encrypted encrypted,
      @JsonProperty("checksum") String checksum,
      @JsonProperty("createdBy") String createdBy,
      @JsonProperty("createdAt") Long createdAt
  ) {
    this.path = path;
    this.key = key;
    this.version = version;
    this.encrypted = encrypted;
    this.checksum = checksum;
    this.createdBy = createdBy;
    this.createdAt = createdAt;
  }

  @JsonProperty("path")
  public String getPath() {
    return path;
  }

  @JsonProperty("path")
  public void setPath(String path) {
    this.path = path;
  }

  @JsonProperty("key")
  public String getKey() {
    return key;
  }

  @JsonProperty("key")
  public void setKey(String key) {
    this.key = key;
  }

  @JsonProperty("version")
  public Integer getVersion() {
    return this.version;
  }

  @JsonProperty("version")
  public void setVersion(Integer version) {
    this.version = version;
  }

  @JsonProperty("encrypted")
  public Encrypted getEncrypted() {
    return this.encrypted;
  }

  @JsonProperty("encrypted")
  public void setEncrypted(Encrypted encrypted) {
    this.encrypted = encrypted;
  }

  @JsonProperty("checksum")
  public String getChecksum() {
    return checksum;
  }

  @JsonProperty("checksum")
  public void setChecksum(String checksum) {
    this.checksum = checksum;
  }

  @JsonProperty("createdBy")
  public String getCreatedBy() {
    return createdBy;
  }

  @JsonProperty("createdBy")
  public void setCreatedBy(String createdBy) {
    this.createdBy = createdBy;
  }

  @JsonProperty("createdAt")
  public Long getCreatedAt() {
    return createdAt;
  }

  @JsonProperty("createdAt")
  public void setCreatedAt(Long createdAt) {
    this.createdAt = createdAt;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    SecretValue that = (SecretValue) o;
    return Objects.equals(path, that.path)
        && Objects.equals(key, that.key)
        && Objects.equals(version, that.version)
        && Objects.equals(encrypted, that.encrypted)
        && Objects.equals(checksum, that.checksum)
        && Objects.equals(createdBy, that.createdBy)
        && Objects.equals(createdAt, that.createdAt);
  }

  @Override
  public int hashCode() {
    return Objects.hash(path, key, version, encrypted, checksum, createdBy, createdAt);
  }

  @Override
  public String toString() {
    return "SecretValue{"
        + "path='" + path + '\''
        + ", key='" + key + '\''
        + ", version=" + version
        + ", encrypted=" + encrypted
        + ", checksum='" + checksum + '\''
        + ", createdBy='" + createdBy + '\''
        + ", createdAt=" + createdAt
        + '}';
  }

  @Override
  public int compareTo(SecretValue that) {
    if (this.path.compareTo(that.path) < 0) {
      return -1;
    } else if (this.path.compareTo(that.path) > 0) {
      return 1;
    }

    if (compareKeys(this.key, that.key) < 0) {
      return -1;
    } else if (compareKeys(this.key, that.key) > 0) {
      return 1;
    }

    if (this.version.compareTo(that.version) < 0) {
      return -1;
    } else if (this.version.compareTo(that.version) > 0) {
      return 1;
    }

    if (this.checksum.compareTo(that.checksum) < 0) {
      return -1;
    } else if (this.checksum.compareTo(that.checksum) > 0) {
      return 1;
    }

    return 0;
  }
}
